@@ -24,9 +24,9 @@ from .htmlprocessing import (convert_tags, discard_unwanted,
 from .metadata import extract_metadata, METADATA_LIST
 from .settings import (MIN_EXTRACTED_SIZE, MIN_EXTRACTED_COMM_SIZE,
                        MIN_OUTPUT_SIZE, MIN_OUTPUT_COMM_SIZE, TAG_CATALOG)
-from .utils import load_html, trim, txttocsv
-from .xml import (add_xml_meta, build_json_output, build_xml_output,
-                  build_tei_output, control_xml_output, xmltotxt)
+from .utils import load_html, trim, txttocsv, is_image_file
+from .xml import (build_json_output, build_xml_output, build_tei_output,
+                  control_xml_output, xmltotxt)
 from .xpaths import BODY_XPATH, COMMENTS_XPATH
 
 
@@ -236,6 +236,24 @@ def handle_table(table_elem):
     return None
 
 
+def handle_image(element):
+    '''Process image element'''
+    if not 'data-src' in element.attrib and not 'src' in element.attrib:
+        return None
+    # image source
+    processed_element = etree.Element(element.tag)
+    if element.get('data-src') is not None and is_image_file(element.get('data-src')):
+        processed_element.set('src', element.get('data-src'))
+    elif element.get('src') is not None and is_image_file(element.get('src')):
+        processed_element.set('src', element.get('src'))
+    # additional data
+    if element.get('alt') is not None:
+        processed_element.set('alt', element.get('alt'))
+    if element.get('title') is not None:
+        processed_element.set('title', element.get('title'))
+    return processed_element
+
+
 def recover_wild_paragraphs(tree, result_body, potential_tags=TAG_CATALOG, deduplicate=True):
     '''Look for all p-elements, including outside of the determined frame
        and throughout the document to recover potentially missing text parts'''
@@ -270,6 +288,8 @@ def handle_textelem(element, potential_tags, dedupbool):
         new_element = handle_formatting(element)
     elif element.tag == 'table' and 'table' in potential_tags:
         new_element = handle_table(element)
+    elif element.tag == 'image':
+        new_element = handle_image(element)
     else:
         # other elements (div, ??, ??)
         new_element = handle_other_elements(element, potential_tags, dedupbool)
@@ -492,37 +512,39 @@ def baseline(filecontent):
     return postbody, temp_text, len(temp_text)
 
 
-def determine_returnstring(docmeta, postbody, commentsbody, output_format, tei_validation):
+def determine_returnstring(docmeta, output_format, tei_validation):
     '''Convert XML tree to chosen format, clean the result and output it as a string'''
     # XML (TEI) steps
     if 'xml' in output_format:
         # last cleaning
-        for element in postbody.iter():
-            if len(element) == 0 and not element.text and not element.tail:
+        for element in docmeta['body'].iter():
+            if element.tag != 'image' and len(element) == 0 and not element.text and not element.tail:
                 parent = element.getparent()
                 if parent is not None:
                     parent.remove(element)
         # build output trees
         if output_format == 'xml':
-            output = build_xml_output(postbody, commentsbody)
-            output = add_xml_meta(output, docmeta)
+            output = build_xml_output(docmeta)
         elif output_format == 'xmltei':
-            output = build_tei_output(postbody, commentsbody, docmeta)
+            output = build_tei_output(docmeta)
         # can be improved
         returnstring = control_xml_output(output, output_format, tei_validation, docmeta)
-    # CSV. JSON and TXT output
+    # CSV, JSON and TXT output
     else:
         if output_format == 'csv':
-            posttext = xmltotxt(postbody)
-            if commentsbody is not None:
-                commentstext = xmltotxt(commentsbody)
+            posttext = xmltotxt(docmeta['body'])
+            if docmeta['commentsbody'] is not None:
+                commentstext = xmltotxt(docmeta['commentsbody'])
             else:
                 commentstext = ''
             returnstring = txttocsv(posttext, commentstext, docmeta)
         elif output_format == 'json':
-            returnstring = build_json_output(docmeta, postbody, commentsbody)
+            returnstring = build_json_output(docmeta)
         else:  # txt
-            returnstring = xmltotxt(build_xml_output(postbody, commentsbody))
+            returnstring = xmltotxt(docmeta['body'])
+            if docmeta['commentsbody'] is not None:
+                returnstring += '\n' + xmltotxt(docmeta['commentsbody'])
+                returnstring = returnstring.strip()
     return returnstring
 
 
@@ -541,11 +563,13 @@ def map_format(output_format, csv_output, json_output, xml_output, tei_output):
 
 
 def bare_extraction(filecontent, url=None, no_fallback=False,
-                    include_comments=True, output_format='txt', target_language=None,
-                    include_tables=True, include_formatting=False, deduplicate=False,
+                    include_comments=True, output_format='python', target_language=None,
+                    include_tables=True, include_images=False, include_formatting=False, deduplicate=False,
                     date_extraction_params=None, with_metadata=False, max_tree_size=None,
-                    url_blacklist=set()):
-    '''Main process for text extraction returning Python variables'''
+                    url_blacklist=None):
+    '''Internal function for text extraction returning bare Python variables'''
+    if url_blacklist is None:
+        url_blacklist = set()
     try:
         # load data
         tree = load_html(filecontent)
@@ -573,10 +597,10 @@ def bare_extraction(filecontent, url=None, no_fallback=False,
             docmeta = dict.fromkeys(METADATA_LIST)
 
         # clean + use LXML cleaner
-        cleaned_tree = tree_cleaning(tree, include_tables)
+        cleaned_tree = tree_cleaning(tree, include_tables, include_images)
 
         # convert tags, the rest does not work without conversion
-        cleaned_tree = convert_tags(cleaned_tree, include_formatting, include_tables)
+        cleaned_tree = convert_tags(cleaned_tree, include_formatting, include_tables, include_images)
 
         # comments first, then remove
         if include_comments is True:
@@ -626,24 +650,35 @@ def bare_extraction(filecontent, url=None, no_fallback=False,
 
     except ValueError:
         LOGGER.info('discarding data for url: %s', url) # docmeta['url'] , record_id
-        return None, None, None, None
-    return docmeta, temp_text, postbody, commentsbody
+        return None
+
+    # special case: python variables
+    if output_format == 'python':
+        docmeta['text'] = xmltotxt(postbody)
+        if include_comments is True:
+            docmeta['comments'] = xmltotxt(commentsbody)
+    else:
+        docmeta['raw-text'], docmeta['body'], docmeta['commentsbody'] = temp_text, postbody, commentsbody
+    return docmeta
 
 
 def extract(filecontent, url=None, record_id=None, no_fallback=False,
             include_comments=True, output_format='txt',
             csv_output=False, json_output=False, xml_output=False, tei_output=False,
             tei_validation=False, target_language=None,
-            include_tables=True, include_formatting=False, deduplicate=False,
-            date_extraction_params=None, with_metadata=False, max_tree_size=None, url_blacklist=set()):
-    '''Wrapper for text extraction and conversion to chosen output format'''
+            include_tables=True, include_images=False, include_formatting=False, deduplicate=False,
+            date_extraction_params=None, with_metadata=False, max_tree_size=None, url_blacklist=None):
+    '''Function exposed by the package:
+       wrapper for text extraction and conversion to chosen output format'''
     # metadata mapping for compatibility
     output_format = map_format(output_format, csv_output, json_output, xml_output, tei_output)
+    if url_blacklist is None:
+        url_blacklist = set()
     # extraction
-    docmeta, temp_text, postbody, commentsbody = bare_extraction(
+    docmeta = bare_extraction(
         filecontent, url=url, no_fallback=no_fallback,
         include_comments=include_comments, output_format=output_format,
-        target_language=target_language, include_tables=include_tables,
+        target_language=target_language, include_tables=include_tables, include_images=include_images,
         include_formatting=include_formatting, deduplicate=deduplicate,
         date_extraction_params=date_extraction_params, with_metadata=with_metadata,
         max_tree_size=max_tree_size, url_blacklist=url_blacklist
@@ -654,9 +689,9 @@ def extract(filecontent, url=None, record_id=None, no_fallback=False,
         # add record ID to metadata
         docmeta['id'] = record_id
         # calculate fingerprint
-        docmeta['fingerprint'] = content_fingerprint(temp_text)
+        docmeta['fingerprint'] = content_fingerprint(docmeta['raw-text'])
     # return
-    return determine_returnstring(docmeta, postbody, commentsbody, output_format, tei_validation)
+    return determine_returnstring(docmeta, output_format, tei_validation)
 
 
 # for legacy and backwards compatibility
