@@ -8,23 +8,23 @@ Listing a series of settings that are applied module-wide.
 
 from multiprocessing import cpu_count
 
-from trafilatura import __version__
+from . import __version__
 
 
-USER_AGENT = 'trafilatura/' + __version__ + '(+https://github.com/adbar/trafilatura)'
+USER_AGENT = 'trafilatura/' + __version__ + ' (+https://github.com/adbar/trafilatura)'
 
 # sanity checks
 MAX_FILE_SIZE = 20000000
 MIN_FILE_SIZE = 10
 SLEEP_TIME = 2
 DOWNLOAD_THREADS = 5
+TIMEOUT = 30
 
 
 # extraction config
 MIN_EXTRACTED_SIZE = 200
 MIN_EXTRACTED_COMM_SIZE = 10
-MIN_DUPLCHECK_SIZE = 100
-MIN_OUTPUT_SIZE = 25
+MIN_OUTPUT_SIZE = 10
 MIN_OUTPUT_COMM_SIZE = 10
 
 
@@ -37,6 +37,7 @@ PROCESSING_TIMEOUT = 30
 
 # deduplication
 LRU_SIZE = 65536
+MIN_DUPLCHECK_SIZE = 100
 MAX_REPETITIONS = 2
 
 
@@ -62,7 +63,11 @@ MANUALLY_CLEANED = [
 ]
 # 'meta', 'hr', 'img', 'data'
 
-MANUALLY_STRIPPED = ['abbr', 'acronym', 'address', 'bdi', 'bdo', 'big', 'cite', 'data', 'dfn', 'font', 'hgroup', 'img', 'ins', 'mark', 'meta', 'ruby', 'small', 'tbody', 'tfoot', 'thead']
+MANUALLY_STRIPPED = [
+    'abbr', 'acronym', 'address', 'bdi', 'bdo', 'big', 'cite', 'data', 'dfn',
+    'font', 'hgroup', 'img', 'ins', 'mark', 'meta', 'ruby', 'small', 'tbody',
+    'tfoot', 'thead',
+]
 # 'center', 'rb', 'wbr'
 
 TAG_CATALOG = frozenset(['blockquote', 'code', 'del', 'fw', 'head', 'hi', 'lb', 'list', 'p', 'pre', 'quote'])
