@@ -6,6 +6,7 @@ Module bundling functions related to HTML and text processing.
 ## under GNU GPL v3 license
 
 # import csv
+import gzip
 import logging
 import re
 import socket
@@ -18,8 +19,6 @@ try:
     import cchardet
 except ImportError:
     cchardet = None
-# https://charset-normalizer.readthedocs.io/en/latest/
-# https://ftfy.readthedocs.io/en/latest/
 
 import requests
 #import urllib3
@@ -27,7 +26,10 @@ import requests
 from lxml import etree, html
 # from lxml.html.soupparser import fromstring as fromsoup
 
-from .settings import MAX_FILE_SIZE, MIN_FILE_SIZE, USER_AGENT
+from requests.adapters import HTTPAdapter
+from requests.packages.urllib3.util.retry import Retry
+
+from .settings import MAX_FILE_SIZE, MIN_FILE_SIZE, TIMEOUT, USER_AGENT
 
 
 LOGGER = logging.getLogger(__name__)
@@ -37,23 +39,48 @@ HEADERS = {
     # 'Connection': 'close',  # another way to cover tracks
     'User-Agent': USER_AGENT,  # your string here
 }
+RETRY_STRATEGY = Retry(
+    total=3,
+    backoff_factor=TIMEOUT*2,
+    status_forcelist=[429, 500, 502, 503, 504],
+)
+ADAPTER = HTTPAdapter(max_retries=RETRY_STRATEGY)
 # urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 SESSION = requests.Session()
+SESSION.mount("https://", ADAPTER)
+SESSION.mount("http://", ADAPTER)
 SESSION.headers.update(HEADERS)
 
 # collect_ids=False, default_doctype=False, huge_tree=True,
 HTML_PARSER = html.HTMLParser(remove_comments=True, remove_pis=True, encoding='utf-8')
 RECOVERY_PARSER = html.HTMLParser(remove_comments=True, remove_pis=True)
 
-UNICODE_WHITESPACE = re.compile(r'\u00A0|\u1680|\u2000|\u2001|\u2002|\u2003|\u2004|\u2005|\u2006|\u2007|\u2008|\u2009|\u200a|\u2028|\u2029|\u202F|\u205F|\u3000')
+UNICODE_WHITESPACE = re.compile(
+    r'''
+    \u00A0|\u1680|\u2000|\u2001|\u2002|\u2003|\u2004|\u2005|\u2006|\u2007|
+    \u2008|\u2009|\u200a|\u2028|\u2029|\u202F|\u205F|\u3000
+    '''
+)
 
 NO_TAG_SPACE = re.compile(r'(?<![p{P}>])\n')
 SPACE_TRIMMING = re.compile(r'\s+', flags=re.UNICODE|re.MULTILINE)
 
 NOPRINT_TRANS_TABLE = {
-    i: None for i in range(0, sys.maxunicode + 1) if not chr(i).isprintable() and not chr(i) in (' ', '\t', '\n')
+    i: None for i in range(0, sys.maxunicode + 1)
+    if not chr(i).isprintable() and not chr(i) in (' ', '\t', '\n')
 }
-# .isspace() # unicodedata.category(char)[0] != "C" or char in ('\t', '\n')
+
+# Regex to check image file extensions
+IMAGE_EXTENSION = re.compile(r'([^\s]+(\.(jpe?g|png|gif|bmp)))')
+
+# Regex for crude extraction of host/domain name
+HOSTINFO = re.compile(r'https?://[^/]+')
+
+
+def is_gz_file(contents):
+    """Tell if a file's magic number corresponds to the GZip format"""
+    # source: https://stackoverflow.com/questions/3703276/how-to-tell-if-a-file-is-gzip-compressed
+    return contents[:2] == b'\x1f\x8b'
 
 
 def isutf8(data):
@@ -84,23 +111,30 @@ def detect_encoding(bytesobject):
 
 
 def decode_response(response):
-    """Read the first chunk of server response and decode it"""
-    guessed_encoding = detect_encoding(response.content)
+    """Read the Requests object corresponding to the server response,
+       check if it could be GZip and eventually decompress it, then
+       try to guess its encoding and decode it to return a unicode string"""
+    resp_content, resp_text = response.content, response.text
+    if is_gz_file(response.content):
+        decompressed = gzip.decompress(response.content)
+        resp_content, resp_text = decompressed, \
+            str(decompressed, encoding='utf-8', errors='replace')
+    guessed_encoding = detect_encoding(resp_content)
     LOGGER.debug('response/guessed encoding: %s / %s', response.encoding, guessed_encoding)
     # process
     if guessed_encoding is not None:
         try:
-            htmltext = response.content.decode(guessed_encoding)
+            htmltext = resp_content.decode(guessed_encoding)
         except UnicodeDecodeError:
             LOGGER.warning('encoding error: %s / %s', response.encoding, guessed_encoding)
-            htmltext = response.text
+            htmltext = resp_text
     else:
-        htmltext = response.text
+        htmltext = resp_text
     return htmltext
 
 
 def fetch_url(url):
-    """ Fetch page using requests/urllib3
+    """ Fetch page using requests/urllib3 and decode the response
     Args:
         URL: URL of the page to fetch
     Returns:
@@ -112,25 +146,26 @@ def fetch_url(url):
     try:
         # read by streaming chunks (stream=True, iter_content=xx)
         # so we can stop downloading as soon as MAX_FILE_SIZE is reached
-        response = SESSION.get(url, timeout=30, verify=False, allow_redirects=True, headers=HEADERS)
+        response = SESSION.get(url, timeout=TIMEOUT, verify=False, allow_redirects=True, headers=HEADERS)
     except (requests.exceptions.MissingSchema, requests.exceptions.InvalidURL):
         LOGGER.error('malformed URL: %s', url)
     except requests.exceptions.TooManyRedirects:
         LOGGER.error('redirects: %s', url)
     except requests.exceptions.SSLError as err:
         LOGGER.error('SSL: %s %s', url, err)
-    except (socket.timeout, requests.exceptions.ConnectionError, requests.exceptions.Timeout, socket.error, socket.gaierror) as err:
+    except (socket.timeout, requests.exceptions.ConnectionError,
+            requests.exceptions.Timeout, socket.error, socket.gaierror) as err:
         LOGGER.error('connection: %s %s', url, err)
     #except Exception as err:
     #    logging.error('unknown: %s %s', url, err) # sys.exc_info()[0]
     else:
         # safety checks
         if response.status_code != 200:
-            LOGGER.error('not a 200 response: %s', response.status_code)
+            LOGGER.error('not a 200 response: %s for URL %s', response.status_code, url)
         elif response.text is None or len(response.text) < MIN_FILE_SIZE:
-            LOGGER.error('too small/incorrect: %s %s', url, len(response.text))
+            LOGGER.error('too small/incorrect for URL %s', url)
         elif len(response.text) > MAX_FILE_SIZE:
-            LOGGER.error('too large: %s %s', url, len(response.text))
+            LOGGER.error('too large: length %s for URL %s', len(response.text), url)
         else:
             return decode_response(response)
     return None
@@ -158,7 +193,7 @@ def load_html(htmlobject):
                 try:
                     htmlobject = htmlobject.decode(guessed_encoding)
                     tree = html.fromstring(htmlobject, parser=HTML_PARSER)
-                except UnicodeDecodeError:
+                except (LookupError, UnicodeDecodeError):  # VISCII encoding
                     LOGGER.warning('encoding issue: %s', guessed_encoding)
                     tree = html.fromstring(htmlobject, parser=RECOVERY_PARSER)
         else:
@@ -200,7 +235,9 @@ def txttocsv(text, comments, docmeta):
     text = trim(' '.join(text.splitlines()))
     if comments is not None:
         comments = trim(' '.join(comments.splitlines()))
-    tsv_output = '{url}\t{fingerprint}\t{hostname}\t{doctitle}\t{docdate}\t{text}\t{comments}\n'.format(
+    tsv_output = \
+        '{url}\t{fingerprint}\t{hostname}\t{doctitle}\t{docdate}\t{text}\t{comments}\n' \
+        .format(
         url=docmeta['url'],
         fingerprint=docmeta['fingerprint'],
         hostname=docmeta['hostname'],
@@ -259,3 +296,22 @@ def trim(string):
         return SPACE_TRIMMING.sub(r' ', NO_TAG_SPACE.sub(r' ', string)).strip(' \t\n\r\v')
     except TypeError:
         return None
+
+
+def is_image_file(imagesrc):
+    '''Check if the observed string corresponds to a valid image extension,
+       return False otherwise'''
+    if imagesrc is not None and IMAGE_EXTENSION.search(imagesrc):
+        return True
+    return False
+
+
+def fix_relative_urls(baseurl, url):
+    'Prepend protocol and host information to relative links.'
+    if url.startswith('/'):
+        urlfix = baseurl + url
+    elif not url.startswith('http'):
+        urlfix = baseurl + '/' + url
+    else:
+        urlfix = url
+    return urlfix
