@@ -11,16 +11,17 @@ import logging
 import pickle
 
 from io import StringIO
+from pathlib import Path
+
 from lxml import etree
 
-import pkg_resources
-
+from . import __version__
 from .utils import sanitize
 
 
 LOGGER = logging.getLogger(__name__)
 # validation
-TEI_SCHEMA = pkg_resources.resource_filename('trafilatura', 'data/tei-schema.pickle')
+TEI_SCHEMA = str(Path(__file__).parent / 'data/tei-schema.pickle')
 TEI_VALID_TAGS = {'body', 'cell', 'code', 'del', 'div', 'fw', 'head', 'hi', 'item', \
                   'lb', 'list', 'p', 'quote', 'row', 'table'}
 TEI_VALID_ATTRS = {'rend', 'rendition', 'role', 'type'}
@@ -29,36 +30,39 @@ TEI_RELAXNG = None # to be downloaded later if necessary
 CONTROL_PARSER = etree.XMLParser(remove_blank_text=True)
 
 
-def build_json_output(docmeta, postbody, commentsbody):
+def build_json_output(docmeta):
     '''Build JSON output based on extracted information'''
     outputdict = docmeta
     outputdict['source'] = outputdict.pop('url')
-    outputdict['source-hostname'] = outputdict.pop('hostname')
+    outputdict['source-hostname'] = outputdict.pop('sitename')
     outputdict['excerpt'] = outputdict.pop('description')
     outputdict['categories'] = ';'.join(outputdict['categories'])
     outputdict['tags'] = ';'.join(outputdict['tags'])
-    outputdict['text'] = xmltotxt(postbody)
-    outputdict['comments'] = xmltotxt(commentsbody)
+    outputdict['text'] = xmltotxt(outputdict.pop('body'))
+    if outputdict['commentsbody'] is not None:
+        outputdict['comments'] = xmltotxt(outputdict.pop('commentsbody'))
+    else:
+        del outputdict['commentsbody']
     return json.dumps(outputdict)
 
 
-
 def clean_attributes(tree):
     '''Remove unnecessary attributes'''
     for elem in tree.iter():
-        if elem.tag not in ('del', 'hi'):
+        if elem.tag not in ('del', 'hi', 'image'):
             elem.attrib.clear()
     return tree
 
 
-def build_xml_output(postbody, commentsbody):
+def build_xml_output(docmeta):
     '''Build XML output tree based on extracted information'''
     output = etree.Element('doc')
-    postbody.tag = 'main'
-    output.append(postbody)
-    if commentsbody is not None:
-        commentsbody.tag = 'comments'
-        output.append(commentsbody)
+    output = add_xml_meta(output, docmeta)
+    docmeta['body'].tag = 'main'
+    output.append(docmeta['body'])
+    if docmeta['commentsbody'] is not None:
+        docmeta['commentsbody'].tag = 'comments'
+        output.append(docmeta['commentsbody'])
 # XML invalid characters
 # https://chase-seibert.github.io/blog/2011/05/20/stripping-control-characters-in-python.html
     return clean_attributes(output)
@@ -105,10 +109,10 @@ def add_xml_meta(output, docmeta):
     return output
 
 
-def build_tei_output(postbody, commentsbody, docmeta):
+def build_tei_output(docmeta):
     '''Build TEI-XML output tree based on extracted information'''
     # build TEI tree
-    output = write_teitree(postbody, commentsbody, docmeta)
+    output = write_teitree(docmeta)
     # filter output (strip unwanted elements), just in case
     # check and repair
     output = check_tei(output, docmeta['url'])
@@ -151,7 +155,7 @@ def validate_tei(tei):  # , filename=""
         TEI_RELAXNG = etree.RelaxNG(relaxng_doc)
     result = TEI_RELAXNG.validate(tei)
     if result is False:
-        print(TEI_RELAXNG.error_log.last_error)
+        LOGGER.warning('not a valid TEI document: %s', TEI_RELAXNG.error_log.last_error)
     return result
 
 
@@ -220,7 +224,7 @@ def xmltotxt(xmloutput):
     return sanitize(''.join(returnlist))
 
 
-def write_teitree(postbody, commentsbody, docmeta):
+def write_teitree(docmeta):
     '''Bundle the extracted post and comments into a TEI tree'''
     tei = etree.Element('TEI', xmlns='http://www.tei-c.org/ns/1.0')
     header = etree.SubElement(tei, 'teiHeader')
@@ -231,13 +235,13 @@ def write_teitree(postbody, commentsbody, docmeta):
     textelem = etree.SubElement(tei, 'text')
     textbody = etree.SubElement(textelem, 'body')
     # post
-    postbody = clean_attributes(postbody)
+    postbody = clean_attributes(docmeta['body'])
     postbody.tag = 'div'
     postbody.set('type', 'entry') # rendition='#pst'
     textbody.append(postbody)
     # comments
-    if commentsbody is not None and len(commentsbody) > 0:
-        commentsbody = clean_attributes(commentsbody)
+    if docmeta['commentsbody'] is not None:
+        commentsbody = clean_attributes(docmeta['commentsbody'])
         commentsbody.tag = 'div'
         commentsbody.set('type', 'comments') # rendition='#cmt'
         textbody.append(commentsbody)
@@ -254,13 +258,14 @@ def write_fullheader(header, docmeta):
         bib_author = etree.SubElement(bib_titlestmt, 'author')
         bib_author.text = docmeta['author']
     publicationstmt_a = etree.SubElement(filedesc, 'publicationStmt')
+    # insert an empty paragraph for conformity
     publicationstmt_p = etree.SubElement(publicationstmt_a, 'p')
-    if docmeta['id'] is not None:
-        idno = etree.SubElement(publicationstmt_a, 'idno', type='id')
-        idno.text = docmeta['id']
     notesstmt = etree.SubElement(filedesc, 'notesStmt')
-    note = etree.SubElement(notesstmt, 'note', type='fingerprint')
-    note.text = docmeta['fingerprint']
+    if docmeta['id'] is not None:
+        idno = etree.SubElement(notesstmt, 'note', type='id')
+        idno.text = docmeta['id']
+    fingerprint = etree.SubElement(notesstmt, 'note', type='fingerprint')
+    fingerprint.text = docmeta['fingerprint']
     sourcedesc = etree.SubElement(filedesc, 'sourceDesc')
     source_bibl = etree.SubElement(sourcedesc, 'bibl')
     if docmeta['sitename'] and docmeta['date']:
@@ -310,7 +315,7 @@ def write_fullheader(header, docmeta):
             tags_list.text = ','.join(docmeta['tags'])
     encodingdesc = etree.SubElement(header, 'encodingDesc')
     appinfo = etree.SubElement(encodingdesc, 'appInfo')
-    application = etree.SubElement(appinfo, 'application', version=pkg_resources.get_distribution('trafilatura').version, ident='Trafilatura')
+    application = etree.SubElement(appinfo, 'application', version=__version__, ident='Trafilatura')
     label = etree.SubElement(application, 'label')
     label.text = 'Trafilatura'
     pointer = etree.SubElement(application, 'ptr', target='https://github.com/adbar/trafilatura')
